/*
 * osdanim.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef OSDANIM_H
#define OSDANIM_H

#include <QTime>

#include "osdobject.h"



class OsdAnim : public OsdObject
{
	Q_OBJECT
public:
	enum OsdAnimType{ None=1, LinearHorizontal, LinearVertical, };

	OsdAnim( OsdConfig *conf );
	~OsdAnim();

	void setAnimType( OsdAnimType a ) { type = a; }
	void setAnimDuration( int ms ) { duration = ms; }
	void setHideVector( QPoint p ) { hideVector = p; }
	void setShowVector( QPoint p ) { showVector = p; }
	bool isHidden() { return hidden; }
	void setHidden( bool b ) { hidden = b; }
	bool isMoving() { return moving; }
	void show();
	void hide();
	void setTransform();
	void resetTransform() { glPopMatrix(); }


public slots:
	void toggle();

private:
	bool moving;
	OsdAnimType type;
	bool hidden;
	int duration;
	QPoint showVector, hideVector;
	QPoint *currentVector;
	int offsetX, offsetY;
	QTime startTime;
	int timeOffset;

signals:
	void startRepaint();
	void stopRepaint();
};
#endif // OSDANIM_H
