/*
 * xinethumb.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <KDebug>

#include <QFile>

#include "xinethumb.h"

#define IDLE_TIMEOUT 30000



XineThumb::XineThumb()
{
	program = "kaffeinethumbnailer";
	arguments << "batchMode" << QString::number(START_POS) << QString::number(THUMB_WIDTH);
	proc = 0;
	mrlCounter = 0;

	connect( &idleTimer, SIGNAL(timeout()), this, SLOT(stopProc()) );
}



XineThumb::~XineThumb()
{
	stopProc();
}



void XineThumb::setThumbDir( QString d )
{
	thumbDir.setPath( d );
}



void XineThumb::stopProc()
{
	if ( !proc )
		return;

	disconnect( proc, SIGNAL(started()), this, SLOT(feed()) );
	disconnect( proc, SIGNAL(readyReadStandardError()), this, SLOT(readStderr()) );
	disconnect( proc, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(restartProc(int,QProcess::ExitStatus)) );
	proc->kill();
	delete proc;
	proc = 0;
	mrlCounter = 0;
}



void XineThumb::restartProc( int, QProcess::ExitStatus )
{
	stopProc();
	proc = new QProcess();
	connect( proc, SIGNAL(started()), this, SLOT(feed()) );
	connect( proc, SIGNAL(readyReadStandardError()), this, SLOT(readStderr()) );
	connect( proc, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(restartProc(int,QProcess::ExitStatus)) );
	proc->setReadChannel( QProcess::StandardError );
	kDebug() << "restarting kaffeinethumbnailer";
	proc->start( program, arguments );
	mrlCounter = 0;
	feed();
}



void XineThumb::readStderr()
{
	QString s;
	while ( proc->readLine( buf, 1500 )>0 ) {
		s = buf;
		//kDebug() << s;
		if ( s.contains("unable to open") || s.contains("shot taken") || s.contains("shot failed") ) {
			kDebug() << s;
			--mrlCounter;
			feed();
		}
	}
}



void XineThumb::feed()
{
	QString s;

	if ( !proc )
		restartProc(0,QProcess::NormalExit);

	if ( mrlCounter )
		return;

	while ( mrlList.count() ) {
		s = mrlList.first();
		if ( QFile::exists( thumbDir.filePath(QString(s.remove("/").append(".jpg"))) ) )
			mrlList.removeFirst();
		else
			break;
	}

	if ( mrlList.count() ) {
		s = mrlList.first();
		proc->write( s.append("\n").toLocal8Bit() );
		//kDebug() << "written : " << s;
		s = mrlList.first();
		s = thumbDir.filePath(QString(s.remove("/").append(".jpg")));
		//s = s.remove("/").append(".jpg").prepend("/home/kde4/shot/");
		proc->write( s.append("\n").toLocal8Bit() );
		//kDebug() << "written : " << s;
		mrlList.removeFirst();
		idleTimer.start( IDLE_TIMEOUT );
		++mrlCounter;
	}
}



void XineThumb::prepend( const QStringList &sl )
{
	int i;
	for ( i=sl.count(); i>0; --i )
		mrlList.prepend( sl.at(i-1) );
	feed();
}



void XineThumb::prepend( const QString &mrl )
{
	mrlList.prepend( mrl );
	feed();
}



void XineThumb::append( const QStringList &sl )
{
	mrlList += sl;
	feed();
}



void XineThumb::append( const QString &mrl )
{
	mrlList.append( mrl );
	feed();
}



void XineThumb::resetList()
{
	mrlList.clear();
}
